@@ -16,6 +16,7 @@
 #include "mm-modem-helpers-mbim.h"
 #include "mm-modem-helpers.h"
 #include "mm-enums-types.h"
+#include "mm-flags-types.h"
 #include "mm-errors-types.h"
 #include "mm-error-helpers.h"
 #include "mm-log-object.h"
@@ -128,6 +129,24 @@ mm_modem_3gpp_registration_state_from_mbim_register_state (MbimRegisterState sta
 
 /*****************************************************************************/
 
+MMModem3gppPacketServiceState
+mm_modem_3gpp_packet_service_state_from_mbim_packet_service_state (MbimPacketServiceState state)
+{
+    switch (state) {
+    case MBIM_PACKET_SERVICE_STATE_ATTACHED:
+        return MM_MODEM_3GPP_PACKET_SERVICE_STATE_ATTACHED;
+    case MBIM_PACKET_SERVICE_STATE_ATTACHING:
+    case MBIM_PACKET_SERVICE_STATE_DETACHING:
+    case MBIM_PACKET_SERVICE_STATE_DETACHED:
+        return MM_MODEM_3GPP_PACKET_SERVICE_STATE_DETACHED;
+    case MBIM_PACKET_SERVICE_STATE_UNKNOWN:
+    default:
+        return MM_MODEM_3GPP_PACKET_SERVICE_STATE_UNKNOWN;
+    }
+}
+
+/*****************************************************************************/
+
 MMModemMode
 mm_modem_mode_from_mbim_data_class (MbimDataClass  data_class,
                                     const gchar   *caps_custom_data_class)
@@ -562,6 +581,7 @@ mm_bearer_apn_type_from_mbim_context_type (MbimContextType context_type)
 
 MbimContextType
 mm_bearer_apn_type_to_mbim_context_type (MMBearerApnType   apn_type,
+                                         gboolean          mbim_extensions_supported,
                                          gpointer          log_object,
                                          GError          **error)
 {
@@ -580,8 +600,6 @@ mm_bearer_apn_type_to_mbim_context_type (MMBearerApnType   apn_type,
         return MBIM_CONTEXT_TYPE_IMS;
     if (apn_type & MM_BEARER_APN_TYPE_MMS)
         return MBIM_CONTEXT_TYPE_MMS;
-    if (apn_type &MM_BEARER_APN_TYPE_MANAGEMENT)
-        return MBIM_CONTEXT_TYPE_ADMIN;
     if (apn_type & MM_BEARER_APN_TYPE_VOICE)
         return MBIM_CONTEXT_TYPE_VOICE;
     if (apn_type & MM_BEARER_APN_TYPE_PRIVATE)
@@ -592,14 +610,30 @@ mm_bearer_apn_type_to_mbim_context_type (MMBearerApnType   apn_type,
         return MBIM_CONTEXT_TYPE_VIDEO_SHARE;
     if (apn_type & MM_BEARER_APN_TYPE_LOCAL)
         return MBIM_CONTEXT_TYPE_LOCAL;
-    if (apn_type & MM_BEARER_APN_TYPE_APP)
-        return MBIM_CONTEXT_TYPE_APP;
-    if (apn_type & MM_BEARER_APN_TYPE_XCAP)
-        return MBIM_CONTEXT_TYPE_XCAP;
-    if (apn_type & MM_BEARER_APN_TYPE_TETHERING)
-        return MBIM_CONTEXT_TYPE_TETHERING;
-    if (apn_type & MM_BEARER_APN_TYPE_EMERGENCY)
-        return MBIM_CONTEXT_TYPE_EMERGENCY_CALLING;
+
+    if (mbim_extensions_supported) {
+        if (apn_type & MM_BEARER_APN_TYPE_MANAGEMENT)
+            return MBIM_CONTEXT_TYPE_ADMIN;
+        if (apn_type & MM_BEARER_APN_TYPE_APP)
+            return MBIM_CONTEXT_TYPE_APP;
+        if (apn_type & MM_BEARER_APN_TYPE_XCAP)
+            return MBIM_CONTEXT_TYPE_XCAP;
+        if (apn_type & MM_BEARER_APN_TYPE_TETHERING)
+            return MBIM_CONTEXT_TYPE_TETHERING;
+        if (apn_type & MM_BEARER_APN_TYPE_EMERGENCY)
+            return MBIM_CONTEXT_TYPE_EMERGENCY_CALLING;
+    } else {
+        if ((apn_type & MM_BEARER_APN_TYPE_MANAGEMENT) ||
+            (apn_type & MM_BEARER_APN_TYPE_APP)        ||
+            (apn_type & MM_BEARER_APN_TYPE_XCAP)       ||
+            (apn_type & MM_BEARER_APN_TYPE_TETHERING)  ||
+            (apn_type & MM_BEARER_APN_TYPE_EMERGENCY)) {
+            mm_obj_dbg (log_object,
+                        "MS extensions unsupported: "
+                        "fallback to using default (internet) APN type");
+            return MBIM_CONTEXT_TYPE_INTERNET;
+        }
+    }
 
     str = mm_bearer_apn_type_build_string_from_mask (apn_type);
     g_set_error (error,
@@ -1085,32 +1119,27 @@ select_mbim_signal_with_data_class (MbimDataClass   data_class,
                                     MMSignal      **lte,
                                     MMSignal      **nr5g)
 {
-    switch (data_class) {
-    case MBIM_DATA_CLASS_5G_NSA:
-    case MBIM_DATA_CLASS_5G_SA:
+    if (data_class & (MBIM_DATA_CLASS_5G_NSA |
+                      MBIM_DATA_CLASS_5G_SA))
         return nr5g;
-    case MBIM_DATA_CLASS_LTE:
+    if (data_class & (MBIM_DATA_CLASS_LTE))
         return lte;
-    case MBIM_DATA_CLASS_UMTS:
-    case MBIM_DATA_CLASS_HSDPA:
-    case MBIM_DATA_CLASS_HSUPA:
+    if (data_class & (MBIM_DATA_CLASS_UMTS |
+                      MBIM_DATA_CLASS_HSDPA |
+                      MBIM_DATA_CLASS_HSUPA))
         return umts;
-    case MBIM_DATA_CLASS_GPRS:
-    case MBIM_DATA_CLASS_EDGE:
+    if (data_class & (MBIM_DATA_CLASS_GPRS |
+                      MBIM_DATA_CLASS_EDGE))
         return gsm;
-    case MBIM_DATA_CLASS_1XEVDO:
-    case MBIM_DATA_CLASS_1XEVDO_REVA:
-    case MBIM_DATA_CLASS_1XEVDV:
-    case MBIM_DATA_CLASS_3XRTT:
-    case MBIM_DATA_CLASS_1XEVDO_REVB:
+    if (data_class & (MBIM_DATA_CLASS_1XEVDO |
+                      MBIM_DATA_CLASS_1XEVDO_REVA |
+                      MBIM_DATA_CLASS_1XEVDV |
+                      MBIM_DATA_CLASS_3XRTT |
+                      MBIM_DATA_CLASS_1XEVDO_REVB))
         return evdo;
-    case MBIM_DATA_CLASS_1XRTT:
+    if (data_class & MBIM_DATA_CLASS_1XRTT)
         return cdma;
-    case MBIM_DATA_CLASS_UMB:
-    case MBIM_DATA_CLASS_CUSTOM:
-    default:
-        return NULL;
-    }
+    return NULL;
 }
 
 gboolean
@@ -1131,12 +1160,18 @@ mm_signal_from_mbim_signal_state (MbimDataClass          data_class,
     MMSignal **last_updated = NULL;
     guint      n_out_updated = 0;
 
-    *out_cdma = NULL;
-    *out_evdo = NULL;
-    *out_gsm = NULL;
-    *out_umts = NULL;
-    *out_lte = NULL;
-    *out_nr5g = NULL;
+    if (out_cdma)
+        *out_cdma = NULL;
+    if (out_evdo)
+        *out_evdo = NULL;
+    if (out_gsm)
+        *out_gsm = NULL;
+    if (out_umts)
+        *out_umts = NULL;
+    if (out_lte)
+        *out_lte = NULL;
+    if (out_nr5g)
+        *out_nr5g = NULL;
 
     /* When MBIMEx v2.0 is available, we get LTE+5GNR information reported
      * in the RSRP/SNR list of items. */
@@ -1231,3 +1266,233 @@ mm_signal_from_mbim_signal_state (MbimDataClass          data_class,
 
     return TRUE;
 }
+
+gboolean
+mm_signal_from_atds_signal_response (guint32    rssi,
+                                     guint32    rscp,
+                                     guint32    ecno,
+                                     guint32    rsrq,
+                                     guint32    rsrp,
+                                     guint32    snr,
+                                     MMSignal **out_gsm,
+                                     MMSignal **out_umts,
+                                     MMSignal **out_lte)
+{
+
+    if (rscp <= 96) {
+        *out_umts = mm_signal_new ();
+        mm_signal_set_rscp (*out_umts, -120.0 + rscp);
+    }
+
+    if (ecno <= 49) {
+        if (!*out_umts)
+            *out_umts = mm_signal_new ();
+        mm_signal_set_ecio (*out_umts, -24.0 + ((gdouble) ecno / 2));
+    }
+
+    if (rsrq <= 34) {
+        *out_lte = mm_signal_new ();
+        mm_signal_set_rsrq (*out_lte, -19.5 + ((gdouble) rsrq / 2));
+    }
+
+    if (rsrp <= 97) {
+        if (!*out_lte)
+            *out_lte = mm_signal_new ();
+        mm_signal_set_rsrp (*out_lte, -140.0 + rsrp);
+    }
+
+    if (snr <= 35) {
+        if (!*out_lte)
+            *out_lte = mm_signal_new ();
+        mm_signal_set_snr (*out_lte, -5.0 + snr);
+    }
+
+    /* RSSI may be given for all 2G, 3G or 4G so we detect to which one applies */
+    if (rssi <= 31) {
+        gdouble value;
+
+        value = -113.0 + (2 * rssi);
+        if (*out_lte)
+            mm_signal_set_rssi (*out_lte, value);
+        else if (*out_umts)
+            mm_signal_set_rssi (*out_umts, value);
+        else {
+            *out_gsm = mm_signal_new ();
+            mm_signal_set_rssi (*out_gsm, value);
+        }
+    }
+
+    if (!out_gsm && !out_umts && !out_lte) {
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+/*****************************************************************************/
+
+void
+mm_rf_info_free (MMRfInfo *rf_data)
+{
+    g_free (rf_data);
+}
+
+void
+mm_rfim_info_list_free (GList *rfim_info_list)
+{
+    g_list_free_full (rfim_info_list, (GDestroyNotify) mm_rf_info_free);
+}
+
+GList *
+mm_rfim_info_list_from_mbim_intel_rfim_frequency_value_array (MbimIntelRfimFrequencyValueArray *freq_info,
+                                                              guint                             freq_count,
+                                                              gpointer                          log_object)
+{
+    GList *info_list = NULL;
+    guint i;
+
+    for (i = 0; i < freq_count; i++) {
+        MMRfInfo *info;
+
+        /* If Cell info value indicates radio off, then other parameters are invalid.
+         * So those data will be ignored. */
+        if (freq_info[i]->serving_cell_info == MBIM_INTEL_SERVING_CELL_INFO_RADIO_OFF)
+            continue;
+
+        info = g_new0 (MMRfInfo, 1);
+        info->serving_cell_type = MM_SERVING_CELL_TYPE_UNKNOWN;
+        switch (freq_info[i]->serving_cell_info) {
+            case MBIM_INTEL_SERVING_CELL_INFO_PCELL:
+                info->serving_cell_type = MM_SERVING_CELL_TYPE_PCELL;
+                break;
+            case MBIM_INTEL_SERVING_CELL_INFO_SCELL:
+                info->serving_cell_type = MM_SERVING_CELL_TYPE_SCELL;
+                break;
+            case MBIM_INTEL_SERVING_CELL_INFO_PSCELL:
+                info->serving_cell_type = MM_SERVING_CELL_TYPE_PSCELL;
+                break;
+            case MBIM_INTEL_SERVING_CELL_INFO_SSCELL:
+                info->serving_cell_type = MM_SERVING_CELL_TYPE_SSCELL;
+                break;
+            case MBIM_INTEL_SERVING_CELL_INFO_RADIO_OFF:
+            default:
+                info->serving_cell_type = MM_SERVING_CELL_TYPE_INVALID;
+                break;
+        }
+        info->bandwidth = freq_info[i]->bandwidth;
+        info->center_frequency = freq_info[i]->center_frequency;
+        info_list = g_list_append (info_list, info);
+    }
+
+    return info_list;
+}
+
+typedef struct {
+    guint8  band;
+    gdouble fdl_low;
+    guint32 n_offs_dl;
+    guint32 range_dl1;
+    guint32 range_dl2;
+} LteDlRangeData;
+
+static LteDlRangeData lte_dl_range_data [] = {
+    { 1,  2110,      0,      0,   599 },
+    { 2,  1930,    600,    600,  1199 },
+    { 3,  1805,   1200,   1200,  1949 },
+    { 4,  2110,   1950,   1950,  2399 },
+    { 5,   869,   2400,   2400,  2649 },
+    { 6,   875,   2650,   2650,  2749 },
+    { 7,  2620,   2750,   2750,  3449 },
+    { 8,   925,   3450,   3450,  3799 },
+    { 9,  1844.9, 3800,   3800,  4149 },
+    { 10, 2110,   4150,   4150,  4749 },
+    { 11, 1475.9, 4750,   4750,  4949 },
+    { 12,  728,   5000,   5000,  5179 },
+    { 13,  746,   5180,   5180,  5279 },
+    { 14,  758,   5280,   5280,  5379 },
+    { 17,  734,   5730,   5730,  5849 },
+    { 18,  860,   5850,   5850,  5999 },
+    { 19,  875,   6000,   6000,  6149 },
+    { 20,  791,   6150,   6150,  6449 },
+    { 21, 1495.9, 6450,   6450,  6599 },
+    { 33, 1900,   36000, 36000, 36199 },
+    { 34, 2010,   36200, 36200, 36349 },
+    { 35, 1850,   36350, 36350, 36949 },
+    { 36, 1930,   36950, 36950, 37549 },
+    { 37, 1910,   37550, 37550, 37749 },
+    { 38, 2570,   37750, 37750, 38249 },
+    { 39, 1880,   38250, 38250, 38649 },
+    { 40, 2300,   38650, 38650, 39649 },
+};
+
+static gint
+earfcn_to_band_index (guint32  earfcn,
+                      gpointer log_object)
+{
+    guint i;
+
+    for (i = 0; i < G_N_ELEMENTS (lte_dl_range_data); i++) {
+        if (lte_dl_range_data[i].range_dl1 <= earfcn && lte_dl_range_data[i].range_dl2 >= earfcn) {
+            mm_obj_dbg (log_object, "found matching band index %u for earfcn %u", i, earfcn);
+            return i;
+        }
+    }
+    mm_obj_dbg (log_object, "earfcn %u not matched to any band index", earfcn);
+    return -1;
+}
+
+gdouble
+mm_earfcn_to_frequency (guint32  earfcn,
+                        gpointer log_object)
+{
+    gint i;
+
+    i = earfcn_to_band_index (earfcn, log_object);
+    if (i < 0)
+        return 0.0;
+
+    return 1.0e6 * (lte_dl_range_data[i].fdl_low + 0.1 * (earfcn - lte_dl_range_data[i].n_offs_dl));
+}
+
+typedef struct {
+    guint global_khz;
+    guint range_offset;
+    guint nrarfcn_offset;
+    guint range_first;
+    guint range_last;
+} NrRangeData ;
+
+static NrRangeData nr_range_data [] = {
+    { 5,         0,       0,       0,  599999 },
+    { 15,  3000000,  600000,  600000, 2016666 },
+    { 60, 24250080, 2016667, 2016667, 3279165 },
+};
+
+static gint
+nrarfcn_to_range_index (guint32  nrarfcn,
+                        gpointer log_object)
+{
+    guint i;
+
+    for (i = 0; i < G_N_ELEMENTS (nr_range_data); i++) {
+        if (nr_range_data[i].range_first <= nrarfcn &&  nr_range_data[i].range_last >= nrarfcn) {
+            mm_obj_dbg (log_object, "found matching range index %u for nrarfcn %u", i, nrarfcn);
+            return i;
+        }
+    }
+    mm_obj_dbg (log_object, "nrarfcn %u not matched to any range index", nrarfcn);
+    return -1;
+}
+
+gdouble
+mm_nrarfcn_to_frequency (guint32  nrarfcn,
+                         gpointer log_object)
+{
+    gint i;
+
+    i = nrarfcn_to_range_index (nrarfcn, log_object);
+    if (i < 0)
+        return 0.0;
+
+    return 1.0e3 * (nr_range_data[i].range_offset + nr_range_data[i].global_khz * (nrarfcn - nr_range_data[i].nrarfcn_offset));
+}
